@@ -49,6 +49,35 @@
 #define SHT_GNU_HASH		0x6ffffff6
 #endif
 
+#ifndef DT_MIPS_RLD_VERSION
+#define DT_MIPS_RLD_VERSION	0x70000001
+#define DT_MIPS_TIME_STAMP	0x70000002
+#define DT_MIPS_ICHECKSUM	0x70000003
+#define DT_MIPS_IVERSION	0x70000004
+#define DT_MIPS_FLAGS		0x70000005
+#define DT_MIPS_BASE_ADDRESS	0x70000006
+#define DT_MIPS_CONFLICT	0x70000008
+#define DT_MIPS_LIBLIST		0x70000009
+#define DT_MIPS_LOCAL_GOTNO	0x7000000a
+#define DT_MIPS_CONFLICTNO	0x7000000b
+#define DT_MIPS_LIBLISTNO	0x70000010
+#define DT_MIPS_SYMTABNO	0x70000011
+#define DT_MIPS_UNREFEXTNO	0x70000012
+#define DT_MIPS_GOTSYM		0x70000013
+#define DT_MIPS_HIPAGENO	0x70000014
+#define DT_MIPS_RLD_MAP		0x70000016
+#endif
+
+#ifndef R_MIPS_TLS_DTPMOD32
+#define R_MIPS_TLS_DTPMOD32	38
+#define R_MIPS_TLS_DTPREL32	39
+#define R_MIPS_TLS_TPREL32	47
+#endif
+
+#ifndef R_MIPS_GLOB_DAT
+#define R_MIPS_GLOB_DAT		51
+#endif
+
 struct prelink_entry;
 struct prelink_info;
 struct PLArch;
@@ -81,6 +110,9 @@ typedef struct
   GElf_Addr info_DT_CHECKSUM;
   GElf_Addr info_DT_VERNEED, info_DT_VERDEF, info_DT_VERSYM;
   GElf_Addr info_DT_GNU_HASH;
+  GElf_Addr info_DT_MIPS_LOCAL_GOTNO;
+  GElf_Addr info_DT_MIPS_GOTSYM;
+  GElf_Addr info_DT_MIPS_SYMTABNO;
 #define DT_GNU_PRELINKED_BIT 50
 #define DT_CHECKSUM_BIT 51
 #define DT_VERNEED_BIT 52
@@ -121,6 +153,7 @@ struct PLArch
   int R_COPY;
   int R_JMP_SLOT;
   int R_RELATIVE;
+  int (*arch_adjust) (DSO *dso, GElf_Addr start, GElf_Addr adjust);
   int (*adjust_section) (DSO *dso, int n, GElf_Addr start, GElf_Addr adjust);
   int (*adjust_dyn) (DSO *dso, int n, GElf_Dyn *dyn, GElf_Addr start,
 		     GElf_Addr adjust);
@@ -136,6 +169,7 @@ struct PLArch
 			       GElf_Rel *rel, GElf_Addr reladdr);
   int (*prelink_conflict_rela) (DSO *dso, struct prelink_info *info,
   				GElf_Rela *rela, GElf_Addr relaaddr);
+  int (*arch_prelink_conflict) (DSO *dso, struct prelink_info *info);
   int (*apply_conflict_rela) (struct prelink_info *info, GElf_Rela *rela,
 			      char *buf);
   int (*apply_rel) (struct prelink_info *info, GElf_Rel *rel, char *buf);
@@ -155,7 +189,7 @@ struct PLArch
 #define RTYPE_CLASS_TLS		(8|4)
   int (*reloc_class) (int);
   int (*arch_pre_prelink) (DSO *dso);
-  int (*arch_prelink) (DSO *dso);
+  int (*arch_prelink) (struct prelink_info *info);
   int (*arch_undo_prelink) (DSO *dso);
   int (*undo_prelink_rel) (DSO *dso, GElf_Rel *rel, GElf_Addr reladdr);
   int (*undo_prelink_rela) (DSO *dso, GElf_Rela *rela, GElf_Addr relaaddr);
@@ -177,6 +211,7 @@ struct section_move *init_section_move (DSO *dso);
 void add_section (struct section_move *move, int sec);
 void remove_section (struct section_move *move, int sec);
 int reopen_dso (DSO *dso, struct section_move *move, const char *);
+int adjust_symbol_p (DSO *dso, GElf_Sym *sym);
 int check_dso (DSO *dso);
 int dso_is_rdwr (DSO *dso);
 void read_dynamic (DSO *dso);
@@ -203,21 +238,53 @@ int strtabfind (DSO *dso, int strndx, const char *name);
 int shstrtabadd (DSO *dso, const char *name);
 
 /* data.c */
+
+/* Used for reading consecutive blocks of data from a DSO.  */
+struct data_iterator {
+  /* The DSO that is being read.  */
+  DSO *dso;
+
+  /* The data block that contained the last byte to be read.
+     NULL if no data has been read yet or if the end of the
+     DSO has been reached.  */
+  Elf_Data *data;
+
+  /* The section that contains DATA, when DATA is nonnull.  */
+  int sec;
+
+  /* The address of the next byte.  */
+  GElf_Addr addr;
+
+  /* The offset of the next byte from the start of SEC, when DATA
+     is nonnull.  */
+  GElf_Addr sec_offset;
+};
+
 unsigned char * get_data (DSO *dso, GElf_Addr addr, int *scnp);
 #define READWRITEPROTO(le,nn)					\
 uint##nn##_t buf_read_u##le##nn (unsigned char *data);		\
 uint##nn##_t read_u##le##nn (DSO *dso, GElf_Addr addr);		\
 void buf_write_##le##nn (unsigned char *data, uint##nn##_t val);\
 int write_##le##nn (DSO *dso, GElf_Addr addr, uint##nn##_t val);
+#define READWRITEPROTOSIZE(nn)					\
+READWRITEPROTO(le,nn)						\
+READWRITEPROTO(be,nn)						\
+uint##nn##_t buf_read_une##nn (DSO *dso, unsigned char *data);	\
+uint##nn##_t read_une##nn (DSO *dso, GElf_Addr addr);		\
+void buf_write_ne##nn (DSO *dso, unsigned char *data,		\
+		       uint##nn##_t val);			\
+void write_ne##nn (DSO *dso, GElf_Addr addr, uint##nn##_t val);
 READWRITEPROTO(,8)
-READWRITEPROTO(le,16)
-READWRITEPROTO(be,16)
-READWRITEPROTO(le,32)
-READWRITEPROTO(be,32)
-READWRITEPROTO(le,64)
-READWRITEPROTO(be,64)
+READWRITEPROTOSIZE(16)
+READWRITEPROTOSIZE(32)
+READWRITEPROTOSIZE(64)
 #undef READWRITEPROTO
+#undef READWRITEPROTOSIZE
 const char * strptr (DSO *dso, int sec, off_t offset);
+void init_data_iterator (struct data_iterator *it, DSO *dso, GElf_Addr addr);
+unsigned char *get_data_from_iterator (struct data_iterator *it,
+				       GElf_Addr size);
+int get_sym_from_iterator (struct data_iterator *it, GElf_Sym *sym);
 
 #define PL_ARCH \
 static struct PLArch plarch __attribute__((section("pl_arch"),used))
