@@ -14,5 +14,7 @@ int main (void)
   printf ("  for (x = 0; x < 16384; ++x)\n");
   printf ("    if (testzero[x]) abort ();\n");
   printf ("  exit (0);\n}\n\n");
-  printf ("asm (\".section nonalloced,\\\"aw\\\",@nobits; testzero: .skip 16384\");\n");
+  printf ("asm (\".section nonalloced,\\\"aw\\\",@nobits\\n\\t\"\n");
+  printf ("     \".globl testzero\\n\\t\"\n");
+  printf ("     \"testzero: .skip 16384\");\n");
 }
